<style>
  .text { padding: 2px 10px 2px 10px; border: thin solid white; color: bisque; margin-left:25px;  width: fit-content}
  .line { height: 2px; border-width: 0; color: grey; background-color: goldenrod;}
</style>

# Go and Python Functional Testing
This folder contains functional tests (written in go and python) that work with an active server and real database, 
not with mocks functions.

Unit tests are maintained in the same location as the go source and test core functions using a mock server.  Any input
validation tests should be placed there ("move testing leftward") and only workflow tests should be here.

<hr class="line">

## 1. Go Tests
* app_test.go
* test_client.go
* wavefront_test_server.go

## 2. Python Functional Tests
The functional tests are designed to use `pytest` and are broken into several suites:
* **basic**: A simple "smoke test" of the API with mainly "happy path" tests
* **extensive**: Negative testing of the APIs, or complex workflows
* **internal**: Tests that verify the integrity of the python test harness, "unit tests" for the functionals, as it were
* **performance**: Spot tests designed to run as needed locally since they can possibly take a long time

> _**NOTE:** Only those operations that that require a "live" system, or testing otherwise cannot easily be done with the go unit tests
> should be added as functional tests.  Always strive to move testing "leftward" ( 'unit' >> 'functional' >> 'integration') as this
> allows you to find bugs earlier in the development cycle._

Executing the python functionals (under fish shell) once the service is running is as simple as:
1) Enter `+infra -n` to start the infra without log tailing (add `-d` to also clear namespace); infra is ready when you see:
   <pre class="text">
   Deployments stabilized in 1.274 second
   You can also run [skaffold run --tail] to get the logs
   </pre>

2) Enter `+app` to start the app; app is ready when you see:
   <pre class="text">
   Deployments stabilized in 6.195 seconds
   Port forwarding deployment/ems-device-hints in namespace local-apps, remote port http -> http://0.0.0.0:5004
   Press Ctrl+C to exit
   [ems-device-hints] 2023-06-28 19:30:06.133293	INFO	app/app.go:83	Golang runtime version: go1.20.5	{"commit_sha": "1.0.0"}
   [ems-device-hints] 2023-06-28 19:30:06.136482	INFO	app/app.go:84	Hinting-api for local listening on [::]:5004
   [ems-device-hints] 	{"commit_sha": "1.0.0"}
   [ems-device-hints] 2023-06-28 19:30:06.136519	INFO	app/app.go:85	Log level: info	{"commit_sha": "1.0.0"}
   </pre>

3) In a different shell, source `orient.fish` and move to the suites folder (`,suite`)
4) Enter `pytest -k 'basic or extensive or internal'`

> _**NOTE:** It is also possible to run the functional tests without the infra, the same way the pipeline does,
> see [**DEPLOYMENT.md**](./functional/DEPLOYMENT.md)_

