// Copyright (c) 2023 VMware Carbon Black, all rights reserved

// Package test -- Functional testing of the hinting service
package test

import (
	"encoding/json"
	"fmt"
	"os"
	"regexp"
	"strconv"
	"sync/atomic"
	"testing"
	"time"

	"gitlab.bit9.local/psc/ems/hinting/internal/app"
	"gitlab.bit9.local/psc/ems/hinting/internal/config"

	"gitlab.bit9.local/psckit/pschttp"
)

// NOTE: Unused for now, until race condition figured out
//
//goland:noinspection GoUnusedFunction
// func testMetrics(wfServer *testWFServer, t *testing.T) {
//	metricCount := atomic.LoadUint64(&wfServer.MetricLineCount)
//	if metricCount == 0 {
//		t.Fatalf("Didn't get any metrics!")
//	}
//	t.Logf("Received metrics: %d", metricCount)
//}

func Test_App(t *testing.T) {
	wfServer := startWFTestServer()
	defer wfServer.Close()
	rbacServer := startRBACTestServer()
	defer rbacServer.Close()

	urlPort := regexp.MustCompile(`(https?://.*):(\d*)/?(.*)`)
	groups := urlPort.FindSubmatch([]byte(rbacServer.URL))
	port, _ := strconv.ParseInt(string(groups[2]), 10, 64)
	userContextHeader := pschttp.UserContext{}
	userContextHeaderJSON, _ := json.Marshal(userContextHeader)
	requestContextHeader := pschttp.RequestContext{
		GatewayHost:           "localhost",
		GatewayPrivilegedPort: int(port),
	}
	requestContextHeaderJSON, _ := json.Marshal(requestContextHeader)

	cfg, err := config.CreateConfigFromFile(".", ".testenv", "env")
	if err != nil || cfg == nil {
		t.Fatalf("Error loading app cfg")
	}
	// Commenting out due to wavefront concurrency issue
	// cfg.WavefrontEndpoint = wfServer.URL
	cfg.MetricInterval = 150 * time.Millisecond
	cfg.ProfilingEnabled = true
	controlSignals := make(chan os.Signal, 1)
	hintingAPI := app.NewHintingServiceFromCfg(cfg, controlSignals)
	go hintingAPI.RunForever()
	count := 0
	for atomic.LoadInt32(hintingAPI.Ready) != 1 && count < 10 {
		time.Sleep(1 * time.Second)
		count++
	}

	//goland:noinspection HttpUrlsUsage
	httpClient := NewHTTPClient(fmt.Sprintf("http://%s:%s", "localhost", hintingAPI.HttpServer.Port),
		HeaderMap{pschttp.UserContextHeader: string(userContextHeaderJSON), pschttp.RequestContextHeader: string(requestContextHeaderJSON)})
	testClient := NewTestClient(t, httpClient)
	testClient.TestPutDeviceLQ()
	testClient.TestPutGroupLQ()
	testClient.TestPutOrgLQ()
	testClient.TestPutEnv()
	testClient.TestHints()
	testClient.TestHintsAudit()

	time.Sleep(1 * time.Second)
	// Commenting out due to wavefront concurrency issue
	// testMetrics(wfServer, t)

	controlSignals <- os.Interrupt
	hintingAPI.Shutdown.Lock()
	hintingAPI.Shutdown.Wait()
}
