<style>
  .op { font-family: monospace; border: thin solid grey; color: chocolate; margin-left:25px; padding: 2px 10px 2px 10px; width: fit-content}
  .text { padding: 2px 10px 2px 10px; border: thin solid white; color: bisque; margin-left:25px;  width: fit-content}
  .line { height: 2px; border-width: 0; color: grey; background-color: goldenrod;}
</style>

# Functional Testing
The `test/functional` folder contains the python environment required to test the example microservices
against either a local kubernetes cluster or against one running on a different host. Tests should be run from 
within Intellij and assume that the instructions located [HERE](../../README.md#3-intellij-project-setup) 
have been performed.

For deployment instructions, see [DEPLOYMENT.md](./DEPLOYMENT.md)

<hr class="line">

## 1. Intellij Testing
To execute from Intellij, you first need two test configurations.

### 1.1 Base Configuration Setups

1. In the menu, select `Run >> Edit Configurations...`
    1. Click the "**+**" to add a new configuration, scroll down and select "**Cloud Code: Kubernetes**"
        1. Change the name to "**Hinting Execution**"
        2. Click the "Environment Variables" icon
           1. Add "**ENV**" with value "**local**"
           2. On M1 macs, add "**DOCKER_HOST**" with a value of "**unix:///Users/_{USER}_/.docker/run/docker.sock**", where {USER} is your username
        3. Scroll down to the "Before launch" section and make sure "**Show this page**" is checked
        4. Scroll back up and check "**Store as project file**", then click "Done"
        5. Click "Apply"
   2. Click the "**+**" to add a new configuration, scroll down to "Python tests" (open if needed) and select "**pytest**"
       1. Change the name to "**Functional Tests**"
       2. "Target" should be "Script path"; click the folder icon in the field below and select the "**functional/suites**" folder
       3. Set "Additional Arguments" to "**--dbg**"
       4. In the "Environment" section, select "Use SDK of module" and choose "**functional**"
       5. For "Working directory" click the folder icon and select the "**functional/suites**" folder
       6. Scroll down to the "Before launch" section and make sure "Show this page" is checked
       7. Scroll back up and check "Store as project file", then click "Done"
       8. Click "Ok"

> _**NOTE:** The "Store as project file" will create a file within a top level ".run" folder.  This folder can be copied
> to be saved in the case you need to re-make the project folder, then copied back, retaining any run configurations._

Once these configurations are made, more targeted test configurations (i.e. for specific test classes) 
can be made by duplicating the "Functional Tests" configuration and making needed changes to name, target (i.e. choose
a specific class instead of the entire folder) and additional aruments.  Working directory should always be the
"**functional/suites**" folder.

### 1.2. Test execution
In a terminal window, proceed to the `hinting` folder and enter:
<p class="op">
export ENV=local;make local-infra-start
</p>

This will start the local infrastructure required by the example microservice.  It is all up and running when you see:
<p class="text">
Port forwarding service/example-microservice-infra-redis-endpoint in namespace local-infra, remote port 6379 -> http://0.0.0.0:40002
Port forwarding service/cbd-aurora-postgres-reader-endpoint in namespace local-infra, remote port 5432 -> http://0.0.0.0:40001
Press Ctrl+C to exit
</p>

If you get the error
<p class="text">
Error from server (AlreadyExists): namespaces "local-apps" already exists
make: *** [create-namespaces] Error 1
</p>

use 
<p class="op">
export ENV=local;make delete-namespaces;make local-infra-start
</p>

instead (and from now on).

This can also be done from any location by using the `+infra` orient.fish or setup.sh function.

#### 1.2.1. Start the service
Choose your "**Hinting Execution**" configuration and in the menu select `Run >> Run 'Hinting Execution'` to start the 
service running, or select `Run >> Debug 'Hinting Execution'` to start the service in debug mode.  It is ready to go when 
you see this in the console:
<p class="text">
[hinting-api] Please check https://pkg.go.dev/github.com/gin-gonic/gin#readme-don-t-trust-all-proxies for details.
[hinting-api] [GIN-debug] Listening and serving HTTP on :5004
[hinting-api] [GIN] 2022/11/02 - 21:05:36 | 200 |     243.084µs |        10.1.0.1 | GET      "/health_check"
</p>

### 1.2.2. Ensure services are forwarded
Once the infra and app are running use `k9s` to examine the running services.
Move down to the `ems-device-hints-infra-postgres-deploy-tf-postgres` service and enter "shift-f", then select "OK".
This should forward localstack, which will have a "(F)" symbol in the row:

![](../../documentation/images/port-forward.png)

You will also need to set the redis service (ems-device-hints-infra-redis-deploy-tf-redis) to be forwarded in the same way.
You only need to do this once after the infra and app are running.

#### 1.2.3. Run the tests
Choose your "**Functional Tests**" configuration and in the menu select `Run >> Run 'Functional Tests'` to execute
the tests, or select `Run >> Debug 'Functional Tests'` to execute the tests in debug mode.  Enter any "Additional Arguments"
changes required and click "Run" or "Debug" button (as presented).  Those tests should execute properly against the local
services based on our currently defined IP and PORT values.  All tests should pass.

> _**NOTE:** It is likely that to run the functional tests you will need to create a file
> called `envfile` at the top of the repository.  Create a file with this content
> and execute the `update_envfile.py` script to update the PGPASSWORD and REDIS_AUTH_TOKEN
> from the running infra before running the functionals._

```text
REDIS_CLUSTER_ENDPOINT=localhost
PGDATABASE=hinting
PGHOST=localhost
PRODUCT=psc
REGION=
COMMIT_SHA=2.5.0
MOSAIC_20_DEPLOYMENT=
CLUSTER_NAME=local
PGPASSWORD=d0b14f6b28dfc08f4dd97230f04220ae5f0c679be58be79737bb74d21015bbba
PGPORT=5432
PGUSER=cbadmin
POSTGRES_ENV=local
BRANCH_NAME=master
LOG_LEVEL=debug
REDIS_ADDRESS=localhost:40002
REDIS_AUTH_TOKEN=ba6c82ab0d4f23cfb3af55d74db801d28a03b0bf864f48f896afbc933a346c05
PGSSLMODE=disable
 ```
  
##### 1.2.2.1. External Service
If executing against a remote service, or if your service IP and PORT differ from what is normally expected use the 
`--ip` option to specify the host address and `--port` option to specify the port. These options are placed in the 
"**Additional Arguments**" field.
