#!/usr/bin/env fish
# Carbon Black EDR Copyright © 2013-2022 VMware, Inc. All Rights Reserved.
################################################################################
## Online help system function definitions.
## Use the "___comm" and "___info" internal functions to document user-facing
## functions (except for the +help function itself)
################################################################################

## NOTE: Since this is sourced before "notifications.fish", no ___debug tracing is possible in this file!

set -eg ORIENT_HELP_LISTINGS

## Call once at the top of each fish function definition module
function ___help_topic
    set -g ORIENT_HELP_LISTINGS $ORIENT_HELP_LISTINGS "**TOPIC**" $argv
end

## Call once each at the bottom of every  user-facing function definition
function ___comm
    set -g ORIENT_HELP_LISTINGS $ORIENT_HELP_LISTINGS "**COMM**" $argv
end
function ___info
    set -g ORIENT_HELP_LISTINGS $ORIENT_HELP_LISTINGS $argv
end


## define help function to use compiled help info
function +help
    if [ "$ORIENT_HELP_LISTINGS" = "" ]
        echo
        ___warning "No help summary available."
        echo
    end

    set last ""; set spacer ""; echo
    for item in $ORIENT_HELP_LISTINGS
        switch $item
            case "**TOPIC**"
                if [ "$last" = "**INFO**" ]
                    echo
                end
                set_color normal; set_color -u -o
                set last "**TOPIC**"
                set spacer ""
            case "**COMM**"
                set_color normal; set_color $ORIENT_HELP_CODE -o
                set last "**COMM**"
                set spacer ""
            case "*"
                if [ "$item" != "" ]
                    if [ "$last" = "**INFO**" ]
                        set spacer "   "
                    end
                    echo "$spacer$item"
                    set_color normal; set_color $ORIENT_HELP_CODE2 -o
                    if [ "$last" != "**TOPIC**" ]
                        set last "**INFO**"
                    end
                end
        end
    end
    set_color normal
    echo
end


################################################################################
## All defined, clean exit
exit 0