// Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved

package test

import (
	"fmt"
	"os"
	"testing"

	"gitlab.bit9.local/psc/ems/hinting/test"
)

//goland:noinspection HttpUrlsUsage
var httpClient = test.NewHTTPClient(fmt.Sprintf("http://%s", os.Getenv("HOST_HEADER")),
	test.HeaderMap{
		"X-Auth-Token": fmt.Sprintf("%s/%s", os.Getenv("API_KEY"), os.Getenv("CONNECTOR_ID")),
	},
)

// ----- Begin Tests --------------------

func TestDevice(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutDevice()

	// and update existing
	testClient.TestPutDevice()
}

func TestDeviceLQ(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutDeviceLQ()
}

func TestDeviceLQNil(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutDeviceLQNil()
}

// ----------

func TestGroup(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutGroup()

	// and update existing
	testClient.TestPutGroup()
}

func TestGroupLQ(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutGroupLQ()
}

func TestGroupLQNil(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutGroupLQNil()
}

// ----------

func TestOrg(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutOrg()

	// and update existing
	testClient.TestPutOrg()
}

func TestOrgLQ(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutOrgLQ()
}

func TestOrgLQNil(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutOrgLQNil()
}

// ----------

func TestEnv(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)
	testClient.TestPutEnv()

	// and update existing
	testClient.TestPutEnv()
}

// ----------

func TestHints(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)

	// create required elements
	testClient.TestPutEnv()
	testClient.TestPutOrg()
	testClient.TestPutGroup()
	testClient.TestPutDevice()

	// normal hint call
	testClient.TestHints()
	// Abnormal hint call
	testClient.TestHintsNoGroupOrOrg()
}

func TestHintsEmpty(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)

	// create required elements
	testClient.TestPutEnv()
	testClient.TestPutOrg()
	testClient.TestPutGroup()
	testClient.TestPutDevice()

	// normal hint call later than config, should produce no hints
	testClient.TestHintsEmpty()
}

func TestHintsAudit(t *testing.T) {
	testClient := test.NewTestClient(t, httpClient)

	// create required elements
	testClient.TestPutEnv()
	testClient.TestPutOrg()
	testClient.TestPutGroup()
	testClient.TestPutDevice()

	// audit hint call
	testClient.TestHintsAudit()
}
