#!/usr/bin/env bash
#
# Copyright (c) 2023 VMware Carbon Black, all rights reserved
#

_dbg=0
_no_redis=0
_no_service=0
while getopts "drs" options; do
    case "${options}" in
        d) _dbg=1;;
        r) _no_redis=1;;
        s) _no_service=1;;
    esac
done

# handle -d (debugging) flag
if [[ ${_dbg} -eq 1 ]]; then
    set -x
fi


# Start the hinting service
if [[ ${_no_service} -eq 0 ]]; then
    RUN_MIGRATOR=false go run ./cmd/migrator
    go run ./cmd/main.go &
else
     echo "No hinting service started -- presumed to be handled externally"
fi

# Wait for dependent services to come up
wait_for_curl () {
  local TIMEOUTS=0
  local URL=$1
  local SERVICE_NAME=$2
  until curl -Ss -fv "${URL}" || [[ ${TIMEOUTS} -eq 12 ]]; do
    echo "Waiting for ${SERVICE_NAME} to come up...";
    sleep 5;
    ((TIMEOUTS+=1))
  done
  STATUS_CODE=$(curl --write-out %{http_code} --silent --output /dev/null "${URL}")
  if [[ "${STATUS_CODE}" != "200" ]]; then
    printf "\n${SERVICE_NAME} returned a %s status code" "${STATUS_CODE}"
    exit 1
  fi
}

wait_for_curl "http://localhost:5004/hinting/healthCheck?isReady" "hinting"

# Execute any analysis utilities FUTURE: None defined yet
function on_exit {
    echo "DONE!"
}
trap on_exit EXIT


echo ""
echo "################################################################################"
echo "                        Service ready, begin testing"
echo "################################################################################"
echo ""

# Run the integration tests
cd ./test/functional/suites 
pytest --dbg -k "internal or basic or extensive" -m "not utility" --junitxml ./report.xml