#  coding: utf-8
#  Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
################################################################################

import logging

import pytest
from pytest_sguid_plugin.sguid_plugin import SGUID

from lib.test_common import TestCommon

_logger = logging.getLogger(__name__)


# noinspection DuplicatedCode
class TestMiscellaneousApi(TestCommon):
    """
    This class tests miscellaneous endpoints.
    """

    # -------------------------------------------------------------------------------- #
    # ----- Begin Tests -------------------------------------------------------------- #
    # -------------------------------------------------------------------------------- #

    @pytest.mark.sguid(SGUID.psc_platform_ems_device_hints)
    def test_00_get_no_such_endpoint(self, hc_connect):
        """
        Verify that GET with no such endpoint returns the proper response.

        :class: functional
        """
        with pytest.raises(hc_connect.HTTPError):
            hc_connect._get("/no_such_endpoint")
            pytest.fail("Did not see expected exception!")
        # noinspection PyUnreachableCode
        self.verify_response(hc_connect, 404)

    # ------------------------------

    @pytest.mark.sguid(SGUID.psc_platform_ems_device_hints)
    def test_10a_get_health_check(self, hc_connect):
        """
        Verify that GET /hinting/healthCheck works as expected.

        :class: functional
        """
        response = hc_connect.health_check_get()
        self.verify_response(hc_connect, 200)
        assert response

    @pytest.mark.sguid(SGUID.psc_platform_ems_device_hints)
    def test_10b_get_health_check_is_ready(self, hc_connect):
        """
        Verify that GET /hinting/healthCheck?isReady=true works as expected.

        :class: functional
        """
        response = hc_connect.health_check_get(is_ready=True)
        self.verify_response(hc_connect, 200)
        assert response
