#  coding: utf-8
#  Copyright (c) 2023 VMware Carbon Black, all rights reserved
################################################################################

import logging
import time

_logger = logging.getLogger(__name__)


################################################################################
class FunctionalTestConfig(object):
    """
    This singleton class holds runtime configuration settings used by automation.
    If functional tests are run with `xdist` there will be one singleton class for each process.
    """

    # ----- Singleton code
    _instance = None

    def __new__(cls):
        if cls._instance is None:
            cls._instance = super(FunctionalTestConfig, cls).__new__(cls)
        return cls._instance

    # ----- Configuration initialization

    # time the test run started
    AUTOMATION_STARTED = int(time.time())

    # location of tests, to be filled in by conftest
    TEST_ROOT: str = None

    # location of repository root, to be filled in by conftest
    REPO_ROOT: str = None

    # inidication of what we are accessing
    SERVICE_TYPE: str = None

    # Location of the deployed service
    SERVICE_IP: str = None
    SERVICE_PORT: int = None

    # holds a common rest connection usable by all tests
    CONNECTION = None

    # set if we are in debug logging mode
    DEBUG = False

    # cleaned up list of options used.
    OPTIONS_USED = {}

    # set if using multiprocessing
    USING_XDIST = False

    # holds count of processes seen
    PROCESSES = 0

    # Envfile path, for possible data scraping, if needed
    ENVFILE: str = None

    # postgres access configurations
    PGHOST: str = None
    PGPORT: int = None
    PGUSER: str = None
    PGPASS: str = None
    DBNAME: str = None

    # holds postgres connection
    POSTGRES = None

    # redis access configurations
    RHOST = None
    RPASS = None

    # holds redis connection
    RDEDIS = None
