#  coding: utf-8
#  Copyright (c) 2022-2023 VMware Carbon Black, all rights reserved
################################################################################

import logging
import time

import pytest
from psc_orgkey import encode_orgkey

from lib.models.device import Device
from lib.models.enumerations import DeviceChangeType
from lib.test_common import TestCommon

_logger = logging.getLogger(__name__)

ORGKEY = encode_orgkey("local", TestCommon.uid())


# noinspection DuplicatedCode
class TestApiTiming(TestCommon):
    """
    This class tests the administrative endpoints.
    """

    # -------------------------------------------------------------------------------- #
    # ----- Begin Tests -------------------------------------------------------------- #
    # -------------------------------------------------------------------------------- #

    @pytest.mark.utility(reason="functional test utility, not a test")
    def test_01a_device_update_timing(self, hc_connect):
        """
        Small test used to figure response timing for Device() responses.

        :class: internal
        """
        start = time.time()
        data = Device(changes=DeviceChangeType.all())
        dev_id = self.uid()
        for i in range(1000):
            hc_connect.v1_device_update(data, org_key=ORGKEY, device_id=dev_id)
        total = time.time() - start
        _logger.info(f"1000 calls = {total} seconds")

    @pytest.mark.utility(reason="functional test utility, not a test")
    def test_01b_device_update_timing_bad(self, hc_connect):
        """
        Small test used to figure response timing for Device() responses.

        :class: internal
        """
        start = time.time()
        data = {"foobar": "bogus"}
        dev_id = self.uid()
        for i in range(1000):
            try:
                # noinspection PyTypeChecker
                hc_connect.v1_device_update(data, org_key=ORGKEY, device_id=dev_id)
            except hc_connect.HTTPError:
                pass
        total = time.time() - start
        _logger.info(f"Time = {total} seconds")
