// Copyright (c) 2023 VMware Carbon Black, all rights reserved

package test

import (
	"bufio"
	"compress/gzip"
	"net/http"
	"net/http/httptest"
	"sync/atomic"
)

func startWFTestServer() *testWFServer {
	handler := &testWFServer{}
	server := httptest.NewServer(handler)
	handler.httpServer = server
	handler.URL = server.URL
	return handler
}

type testWFServer struct {
	MetricLines     []string
	MetricLineCount uint64
	httpServer      *httptest.Server
	URL             string
}

func (s *testWFServer) ServeHTTP(writer http.ResponseWriter, request *http.Request) {
	newLines, err := decodeMetricLines(request)
	if err != nil {
		writer.WriteHeader(500)
	}
	s.MetricLines = append(s.MetricLines, newLines...)
	atomic.AddUint64(&s.MetricLineCount, 1)
	writer.WriteHeader(200)
}

func decodeMetricLines(request *http.Request) ([]string, error) {
	var metricLines []string
	reader, err := gzip.NewReader(request.Body)
	if err != nil {
		return metricLines, err
	}
	scanner := bufio.NewScanner(reader)
	for scanner.Scan() {
		line := scanner.Text()
		metricLines = append(metricLines, line)
	}
	if scanner.Err() != nil {
		_ = reader.Close()
		return metricLines, scanner.Err()
	}
	return metricLines, reader.Close()
}

func (s *testWFServer) Close() {
	s.httpServer.Close()
}
